ALTER TABLE [dbo].[Environment] ADD [Bucket] VARCHAR (50) AFTER [Description];
GO

 ALTER PROCEDURE [dbo].[GetEnvironment] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT [VCC]				'vcc',
			[Name]					'name',
			[Type]					'type',
			[Description]			'description',
			[FTPDirectory]			'ftpDirectory',
			[ApiHost]				'apiHost',
			[ApiPort]				'apiPort',
			[Bucket]				'bucket',
			[ApiUser]				'apiUser',
			[ApiPassword]			'apiPassword',
			[DbDisabled]			'dbDisabled',
			[DbServer]				'dbServer',
			[DbPort]				'dbPort',
			[DbUser]				'dbUser',
			[DbPassword]			'dbPassword',
			[PbxDisabled]			'pbxDisabled',
			[PbxServer]			    'pbxServer',
			[PbxPort]			    'pbxPort',
			[PbxUser]			    'pbxUser',
			[PbxPassword]			'pbxPassword',
			[CreatedDate]			'createdDate',
			[CreatedByUserId]		'createdByUserId',
			[LastModifiedDate]		'lastModifiedDate',
			[LastModifiedByUserId]	'lastModifiedByUserId'
		FROM [Environment] (NOLOCK)
		WHERE [VCC] = @VCC
			AND [Name] = @Name
	
	SET NOCOUNT OFF
END
GO

ALTER PROCEDURE [dbo].[GetProcessInteractionsTask]
(
	@VCC		VARCHAR(50),
	@ProcessId	VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	BEGIN
		-- ACTUALIZAMOS LOS REGISTROS DE INTERACCIONES SINCRONAS(CALL) Y RETORNAMOS LOS DETALLES DE LAS QUE SE VAN A TRANSCRIBIR

		-- Actualiza en el primer momento que se consulta a estos registros, con el valor 1 en el campo IsProcess. Indica que se encuentra utilizado por un proceso.
		UPDATE [dbo].[Audio] SET
			[dbo].[Audio].[IsProcess] = 1,	-- Indicar que se esta tomando como procesando para que ningun otro procesamiento en parelelo pueda utilizar el registro.
			[dbo].[Audio].[IsCancel] = null, -- Limpiamos el campo isCancel para retomar los audios que fallaron en procesos pasados
			[dbo].[Audio].[ErrorMessage] = NULL
		
		-- Retornamos todas las columnas a listar.
		OUTPUT INSERTED.[VCC]							'vcc',
				PA.[ProcessId]							'processId',
				'CALL'									'interactionType',
				INSERTED.[ID]							'id',
				INSERTED.[Section]						'section',
				INSERTED.[Part]							'part',
				INSERTED.[StartDate]					'startDate',
				INSERTED.[EndDate]						'endDate',
				INSERTED.[Campaign]						'campaign',
				INSERTED.[DispositionCode]				'dispositionCode',
				INSERTED.[Agent]						'agent',
				INSERTED.[Type]							'origin',
				INSERTED.[RelativePath]					'relativePath',
				INSERTED.[FileName]						'fileName',
				INSERTED.[Format]                       'format',
				INSERTED.[MediaInfoTime]                'mediaInfoTime',
				INSERTED.[NeedI6Update]					'needI6Update',
				0										'isDiariced',
				P.[Language]							'language',
				P.[DictionaryId]						'dictionaryId',
				P.Retranscription						'retranscription',
				P.Diarization							'diarization',
				P.ChannelChange							'channelchange',
				E.[Name]								'name',
				E.[Type]								'environmentType',
				E.[Description]							'description',
				E.[Bucket]								'bucket',
				E.[FTPDirectory]						'ftpDirectory',
				E.[ApiHost]								'apiHost',
				E.[ApiPort]								'apiPort',
				E.[ApiUser]								'apiUser',
				E.[ApiPassword]							'apiPassword',
				E.[DbDisabled]							'dbDisabled',
				E.[DbServer]							'dbServer',
				E.[DbPort]								'dbPort',
				E.[DbUser]								'dbUser',
				E.[DbPassword]							'dbPassword',
				E.[PbxDisabled]							'pbxDisabled',
				E.[PbxServer]							'pbxServer',
				E.[PbxPort]								'pbxPort',
				E.[PbxUser]								'pbxUser',
				E.[PbxPassword]							'pbxPassword',
				INSERTED.[DurationTime]					'durationTime',
				INSERTED.[RelativeFileName]				'relativeFileName'
		FROM [dbo].[ProcessAudio] PA (NOLOCK)
		INNER JOIN [dbo].[Audio] A (NOLOCK) 
			ON A.[VCC] = PA.[VCC]
				AND CONCAT(A.[ID], '_', A.[Section], '_', A.[Part]) = CONCAT(PA.[AudioId], '_', PA.[Section], '_', PA.[Part])
		INNER JOIN [dbo].[Process] P (NOLOCK) 
			ON P.[VCC] = A.[VCC]
				AND PA.ProcessId = P.[Name]
		INNER JOIN [dbo].[Environment] E (NOLOCK) 
			ON E.[VCC] = PA.[VCC]
				AND P.[Environment] = E.[Name]
		WHERE A.[VCC] = @VCC
			AND PA.[ProcessId] = @ProcessId
			AND (
				(
					ISNULL(A.[IsProcess], 0) = 0				
					AND 
					ISNULL(A.[IsTranscribed], 0) = 0	-- RETORNAR SOLO LAS INTERACCIONES QUE NO HAYAN SIDO TOMADAS POR OTRO PROCESS
				)
				OR 
				(
					A.[IsProcess] = 1						
					AND 
					A.[IsCancel] = 1
				)
			
				
				
			)					-- RETORNAR SOLO LAS INTERACCIONES QUE HAYAN SIDO TOMADAS Y CANCELADAS POR OTRO PROCESS

		-- ACTUALIZAMOS LOS REGISTROS DE INTERACCIONES ASINCRONAS Y RETORNAMOS LOS DETALLES DE LAS QUE SE VAN A TRANSCRIBIR
		UPDATE [dbo].[AsyncInteraction] 
		SET [dbo].[AsyncInteraction].[IsProcess] = 1,
			[dbo].[AsyncInteraction].[IsCancel] = NULL,
			[dbo].[AsyncInteraction].[ErrorMessage] = NULL
		OUTPUT INSERTED.[VCC]														'vcc',
				P.[Name]															'processId',
				INSERTED.[Channel]													'interactionType',
				INSERTED.[ID]														'id',
				INSERTED.[EndDate]													'endDate',
				INSERTED.[Folder]													'folder',
				INSERTED.[Campaign]													'campaign',
				INSERTED.[DispositionCode]											'dispositionCode',
				ISNULL(INSERTED.[LastTakedAgent], INSERTED.[FirstTakedAgent])		'agent',
				INSERTED.[Initiative]												'origin',
				INSERTED.[TotalMessages]											'totalMessages',
				INSERTED.[RawChar]													'rawChar',
				INSERTED.[ExportedContent]											'exportedContent',
				P.[Language]														'language',
				INSERTED.[FileName]                                                 'fileName',
				'allegro'															'environmentType',
				P.Retranscription													'retranscription'
		FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
		INNER JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
			ON AI.[ID] = PAI.[InteractionId]
				-- AND AI.[EndDate] = PAI.[EndDate]
				AND FORMAT (AI.[EndDate], 'yyyy-MM-dd') = FORMAT (PAI.[EndDate], 'yyyy-MM-dd')
		INNER JOIN [dbo].[Process] P (NOLOCK) 
			ON P.[Name] = PAI.[ProcessId] 
		WHERE AI.[VCC] = @VCC
			AND PAI.[ProcessId] = @ProcessId
			AND (
				(
					ISNULL(AI.[IsProcess], 0) = 0
					AND 
					ISNULL(AI.[IsTranscribed], 0) = 0)	-- RETORNAR SOLO LAS INTERACCIONES QUE NO HAYAN SIDO TOMADAS POR OTRO PROCESS
				OR 
				(
					AI.[IsProcess] = 1
					AND 
					AI.[IsCancel] = 1
				)

				
			)					-- RETORNAR SOLO LAS INTERACCIONES QUE HAYAN SIDO TOMADAS Y CANCELADAS POR OTRO PROCESS
	END

	SET NOCOUNT OFF
END
GO

ALTER PROCEDURE [dbo].[SaveEnvironment] (
	@VCC VARCHAR(50) = NULL,
	@Name VARCHAR(50) = NULL,
	@Type VARCHAR(50) = NULL,
	@Description VARCHAR(200) = NULL,
	@Bucket VARCHAR(50) = NULL,
	@FTPDirectory VARCHAR(MAX) = NULL,
	@ApiHost VARCHAR(50) = NULL,
	@ApiPort VARCHAR(50) = NULL,
	@ApiUser VARCHAR(50) = NULL,
	@ApiPassword VARCHAR(50) = NULL, 		
	@DbDisabled Bit = NULL,
	@DbServer VARCHAR(50) = NULL,
	@DbPort VARCHAR(50) = NULL,				
	@DbUser VARCHAR(50) = NULL,
	@DbPassword VARCHAR(50) = NULL,
	@PbxDisabled BIT = NULL, 				
	@PbxServer VARCHAR(50) = NULL,
	@PbxPort VARCHAR(50) = NULL,
	@PbxUser VARCHAR(50) = NULL,			
	@PbxPassword VARCHAR(50) = NULL,
	@CreatedDate DATETIME = NULL,
	@CreatedByUserId VARCHAR(50) = NULL, 	
	@LastModifiedDate DATETIME = NULL,
	@LastModifiedByUserId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [Environment]
		SET [Type] = @Type,
			[Description] = @Description,
			[Bucket] = @Bucket,
			[FTPDirectory] = @FTPDirectory,
			[ApiHost] = @ApiHost,
			[ApiPort] = @ApiPort,
			[ApiUser] = @ApiUser,
			[ApiPassword] = CASE WHEN @ApiPassword IS NULL THEN [ApiPassword] ELSE @ApiPassword END,
			[DbDisabled] = @DbDisabled,
			[DbServer] = @DbServer,
			[DbPort] = @DbPort,
			[DbUser] = @DbUser,
			[DbPassword] = CASE WHEN @DbPassword IS NULL THEN [DbPassword] ELSE @DbPassword END,
			[PbxDisabled] = @PbxDisabled,
			[PbxServer] = @PbxServer,
			[PbxPort] = @PbxPort,
			[PbxUser] = @PbxUser,
			[PbxPassword] = CASE WHEN @PbxPassword IS NULL THEN [PbxPassword] ELSE @PbxPassword END,
			[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
			[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
			[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
			[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [Name] = @Name

    IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [Environment] (
			[VCC], [Name], [Type], [Bucket], [FTPDirectory],
			[Description], [ApiHost], [ApiPort],
			[ApiUser], [ApiPassword], [DbDisabled],
			[DbServer],[DbPort],[DbUser], 
			[DbPassword], [PbxDisabled], [PbxServer],
			[PbxPort], [PbxUser],[PbxPassword],
			[CreatedDate], [CreatedByUserId], 
			[LastModifiedDate],[LastModifiedByUserId], 
			[TimeStamp]
			)
		VALUES (
			@VCC, @Name, @Type, @Bucket, @FTPDirectory,
			@Description, @ApiHost, @ApiPort,
			@ApiUser, @ApiPassword, @DbDisabled, 
			@DbServer,@DbPort,@DbUser, 
			@DbPassword, @PbxDisabled, @PbxServer,
			@PbxPort, @PbxUser, @PbxPassword,
			@CreatedDate, @CreatedByUserId, 
			@LastModifiedDate, @LastModifiedByUserId, 
			GETUTCDATE()
		)
	END

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO

